from   machine import Pin
from   rp2     import asm_pio, PIO, StateMachine
import time, network,socket
 
wlan = network.WLAN(network.STA_IF)
wlan.active(True)
wlan.connect('some_wifi_ssid','the_wifi_password') # connect to signalk via wifi
#set 3 seconds for time out delay
wlandelay =  time.ticks_ms() + 3000
while time.ticks_ms() < wlandelay:
  if wlan.isconnected():
      if wlan.status() <0  or wlan.status() >=3 :
          break
  machine.idle()
if wlan.status() != 3:
    raise RuntimeError('Wi-Fi connection failed')
    machine.reset()
else:
    print('Connected')
    status = wlan.ifconfig()
    print('ip = ' + status[0])
   # wlan_mac = wlan.config('mac')
   # print("MAC Address:", wlan_mac)  # Show MAC for peering
    ipsubnet=status[0].split(".") #establish the subnet to broadcast UDP Seatalk data.
    bcastip = ipsubnet[0] + "." + ipsubnet[1] + "." +ipsubnet[2] + "." + "255"
    print(bcastip)
sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM) #set up UDP socket for transmission

TX_PIN = 0
RX_PIN = 1
BAUD   = 4800
dgram=""
dgramlength=0
@asm_pio(sideset_init = PIO.OUT_HIGH,
         out_init     = PIO.OUT_HIGH,
         out_shiftdir = PIO.SHIFT_RIGHT,
         fifo_join    = PIO.JOIN_TX)
def PioTx9N1(BITS=9):
    pull()            .side(1)  [7]
    set(x, BITS-1)    .side(0)  [7]
    label("txloop")
    out(pins, 1)
    jmp(x_dec, "txloop")        [6]

@asm_pio(set_init     = PIO.IN_HIGH,
         in_shiftdir  = PIO.SHIFT_RIGHT,
         fifo_join    = PIO.JOIN_RX)
def PioRx9N1(BITS=9):
    wait( 0, pin, 0)
    set(x, BITS-1)              [10]
    label("rxloop")
    in_(pins, 1)
    jmp(x_dec, "rxloop")        [6]
    push()

# Need to initialise RX first if using same pin for TX

rxPin = Pin(RX_PIN, Pin.IN, Pin.PULL_UP)
rx = StateMachine(0, PioRx9N1, freq=BAUD * 8, in_base=rxPin)
rx.active(1)

txPin = Pin(TX_PIN, Pin.OUT, value=1)
tx = StateMachine(1, PioTx9N1, freq=BAUD * 8, sideset_base=txPin, out_base=txPin)
tx.active(1)

def Tx(n):
    print("TX  >  {:>03X}  {:>09b}".format(n, n))
    tx.put(n)

def Rx():
    n = rx.get() >> 23
    #print("RX  <  {:>03X}  {:>09b}".format(n, n))
    return n

def AnyAvailable():
    return rx.rx_fifo()

# Flush any data caused by confguration glitches

time.sleep(0.1)
if AnyAvailable():
    print("Flushing\n")
    while AnyAvailable():
        discard = Rx()
        if not AnyAvailable():
            time.sleep(0.1)
    print("")

def Seatalk_Build(input):
    global dgram,dgramlength
   # print(dlng)
    lng=0
    if input>=0x100: # 9th bit set so must be a command byte
        dgramlength =1
        dgram=hex(input-0x100)[2:] + ","
        dgram=(f"{dgram:0>3}") #pad to two zeros
       
       # print(dgramlength,dgram)
    if dgramlength ==1 and input < 0x100: #length is 1 thus commandbyte received so find out the datagram length from next byte
        lng = input & 0x0F #attribute bye, datagram length is 3 + least significant nibble
        dgramlength=(lng + 3)
        dgram += (f"{hex(input)[2:]:0>2}") + "," #Add attribute byte and set length for this datagram
        #print("1",dgram)
    elif dgramlength > 1: # attribute byte has been set so gather rest of datagram
        dgram += (f"{hex(input)[2:]:0>2}") + ","
        strd=dgram[:-1] # remove last comma
        dlng = len(strd.split(",")) #keep building datagram until it matches the length form attribute byte
        if dlng == dgramlength:
            Seatalk_Out(dgram[:-1]) #There she is.
def Seatalk_Out(dgram):
    dgram="$STALK,"+dgram
    print(dgram) #Here she is.
    sock.sendto(dgram.encode(), (bcastip,4001)) # send it in a format Signalk finds delicious on a port.

while True:
   
    dgrams = Rx() #wait for serial data
    Seatalk_Build(dgrams) #construct a datagram to boradcast.
   
    time.sleep(0.01)
        